//= require <dSS/GetTextHelper>
//= require <dSS/util/Util>
Ext.namespace('dSS');

dSS.ScenesPanel = Ext.extend(Ext.Panel, {
    initComponent: function() {

        var contentPanel = {
            ref: 'contentPanel',
            region: 'center',
            layout: 'card',
            hideBorders: true,
            border: true,
            activeItem: 0,
            items: [
            {
                xtype: 'dssdevicepanel'
            }]
        };

        this.treePanel = new Ext.tree.TreePanel({
            useArrows: true,
            autoScroll: true,
            animate: true,
            enableDD: false,
            containerScroll: true,
            border: false,
            rootVisible: false,
            root: new Ext.tree.AsyncTreeNode(),
            listeners: {
                click: this.handleNodeClick.createDelegate(this)
            }
        });

        this.treePanel.getRootNode().expand();

        var treeHolder = new Ext.Panel({
            title: gt.ngettext("Scene", 2),
            region: 'west',
            layout: 'fit',
            split: true,
            width: 225,
            minSize: 175,
            maxSize: 400,
            tbar: this.buildTopToolbar(),
            items: [this.treePanel]
        });

        Ext.apply(this, {
            layout: 'border',
            border: false,
            bodyBorder: false,
            items: [treeHolder, contentPanel]
        });

        this.updateTree();

        dSS.data.Loader.getZoneStore().addListener('datachanged', this.updateTree, this, {
            buffer: 1000
        });

        dSS.ScenesPanel.superclass.initComponent.apply(this, arguments);

        this.on('activate', function(component) {
            var selModel = this.treePanel.getSelectionModel();
            var node = selModel.getSelectedNode();
            var root = this.treePanel.getRootNode();
            if (node === null) {
                if (root.hasChildNodes()) {
                    root.firstChild.select();
                    this.handleNodeClick(root.firstChild);
                } else {
                  this.updateTree();
                }
            } else {
                this.handleNodeClick(node);
            }
        },
        this);

        this.treePanel.on('contextmenu', function(node, event) {
            event.preventDefault();
            event.stopEvent();

            if (!node.isSelected()) {
                node.select();
                this.handleNodeClick(node);
            }

            if (!node.isLeaf()) {
                return;
            }

            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }

            var menuItem = new Ext.menu.Item({
                text: _("Call scene"),
                icon: 'images/on.png',
                handler: this.callScene.createDelegate(this, [node])
            });
            this.contextMenu.add(menuItem);

            var menuItem = new Ext.menu.Item({
                text: _("Save scene"),
                icon: 'images/disk.png',
                handler: this.saveScene.createDelegate(this, [node])
            });
            this.contextMenu.add(menuItem);

            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
        this);

        this.treePanel.on('click', function(node, event) {
            this.callScene(node);
        },
        this);
    },
    updateTree: function() {
        // for leafs the id is encoded as: zone id - group - scene number
        // when no group is desired, 0 is used
        function generateSubNodes(zoneId) {
            return [
            {
                text: _("Light"),
                id: zoneId + '-' + dSS.config.groups.LIGHT + '-all',
                leaf: false,
                children: [
                {
                    text: _("Light scene") + " 1",
                    id: zoneId + '-' + dSS.config.groups.LIGHT + '-' + dSS.config.scenes.T0_S1,
                    leaf: true
                },
                {
                    text: _("Light scene") + " 2",
                    id: zoneId + '-' + dSS.config.groups.LIGHT + '-' + dSS.config.scenes.T0_S2,
                    leaf: true
                },
                {
                    text: _("Light scene") + " 3",
                    id: zoneId + '-' + dSS.config.groups.LIGHT + '-' + dSS.config.scenes.T0_S3,
                    leaf: true
                },
                {
                    text: _("Light scene") + " 4",
                    id: zoneId + '-' + dSS.config.groups.LIGHT + '-' + dSS.config.scenes.T0_S4,
                    leaf: true
                }]
            },
            {
                text: _("Leave home"),
                id: zoneId + '-all-' + dSS.config.scenes.ABSENT,
                leaf: true
            },
            {
                text: _("Door bell"),
                id: zoneId + '-all-' + dSS.config.scenes.SIG_BELL,
                leaf: true
            }];
        }

        var selModel = this.treePanel.getSelectionModel();
        var selectedNode = selModel.getSelectedNode();
        var selectedPath = '';
        if (selectedNode !== null) {
            selectedPath = selectedNode.getPath();
        }

        var zStore = dSS.data.Loader.getZoneStore();
        var devStore = dSS.data.Loader.getDeviceStore();
        var nodes = [];
        zStore.each(
        function(record) {
            if ((record.get('id') !== 0) && (devStore.find('zoneId', record.get('id')) >= 0)) {
                nodes.push({
                    text: record.get('name') + " - " + record.get('id'),
                    id: record.get('id'),
                    leaf: false,
                    children: generateSubNodes(record.get('id'))
                });
            }
            return true;
        },
        this);
        var root = new Ext.tree.AsyncTreeNode({
            children: nodes,
            id: 'root',
            leaf: false
        });
        this.treePanel.setRootNode(root);

        if (selectedPath.length > 0) {
            this.treePanel.selectPath(selectedPath, null, function(result, node) {
                if (result === false) {
                    if (root.firstChild !== null) {
                        root.firstChild.select();
                    }
                }
            });
        } else {
            if (root.firstChild !== null) {
                root.firstChild.select();
            }
        }

        if (selModel.getSelectedNode() !== null) {
            this.handleNodeClick(selModel.getSelectedNode());
        }

    },
    handleNodeClick: function(node) {
        if (!this.isVisible()) {
            return;
        }

        if (node === null) {
            return;
        }

        // we are on the zone node
        var path = node.getPath().split('/').slice(1);
        if (path.length < 2) {
            return;
        }

        var store = dSS.data.Loader.getDeviceStore();
        store.clearFilter();
        store.filterBy(function(record) {
            if (record.get('zoneId') === parseInt(path[1], 10)) {
                if (path.length === 2) {
                    return true;
                }

                var fid = dSS.util.parseFunctionID(record.get('functionID'));

                var nodeType = path[2].split('-');
                // all leaf node ids will have 3 components: zone-group-scene
                if (nodeType.length < 3) {
                    return false;
                }

                // if the group is set, we sort only by group
                if (nodeType[1] !== 'all') {
                    var group = parseInt(nodeType[1], 10);
                    if (!isNaN(group)) {
                        if (fid[0] === group) {
                            return true;
                        }

                        return false;
                    }
                }

                // at this stage groups do not matter, scene filtering applies
                var scene = parseInt(nodeType[2], 10);
                if (isNaN(scene)) {
                    return false;
                }

                switch (scene) {
                case dSS.config.scenes.ABSENT:
                case dSS.config.scenes.SIG_BELL:
                    return true;
                default:
                    break;
                }
            }
            return false;
        });
    },
    buildTopToolbar: function() {
        return [
            {
                iconCls: 'saveScene',
                handler: this.saveSelectedScene.createDelegate(this),
                tooltip: _("Save selected scene"),
                scope: this
            },
            '->', 
            {
                cls: 'zonePanelMenu',
                menu: [
                {
                    text: _("Reload"),
                    handler: this.reload,
                    scope: this
                },
                {
                    text: _("Rescan"),
                    handler: this.rescan,
                    scope: this
                }]
            }];
    },
    rescan: function() {
        this.ownerCt.zoneBrowser.rescan();
    },
    reload: function() {
        this.ownerCt.zoneBrowser.loadData();
    },

    getZoneSceneGroup: function(id) {
        var ret = {
            "zoneId": null,
            "sceneNumber": null,
            "groupId": null
        };

        var ids = id.split('-');
        if (ids.length < 3) {
            return ret;
        }

        var zoneId = parseInt(ids[0], 10);
        if (isNaN(zoneId)) {
            return ret;
        }

        ret.zoneId = zoneId;

        var groupId = parseInt(ids[1], 10);
        if (isNaN(groupId)) {
            groupId = null;
        }

        ret.groupId = groupId;

        var sceneNumber = parseInt(ids[2], 10);
        if (isNaN(sceneNumber)) {
            sceneNumber = null;
        }

        ret.sceneNumber = sceneNumber;

        return ret;
    },

    callScene: function(node) {
        if ((node === null) || (!node.isLeaf())) {
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);

        if ((ids.zoneId === null) || (ids.sceneNumber === null)) {
            return;
        }

        dSS.util.callSceneOnZone(ids.zoneId, ids.sceneNumber, ids.groupId);
    },
    saveSelectedScene: function() {
        var sel = this.treePanel.getSelectionModel();
        var node = sel.getSelectedNode();
        if ((node === null) || (!node.isLeaf())) {
            Ext.Msg.alert(_("No scene selected"), _("Please select the scene that you want to save"));
            return;
        }

        this.saveScene(node);
    },
    saveScene: function(node) {
        if ((node === null) || (!node.isLeaf())) {
            return;
        }

        var ids = this.getZoneSceneGroup(node.id);

        if ((ids.zoneId === null) || (ids.sceneNumber === null)) {
            return;
        }

        dSS.util.saveSceneOnZone(ids.zoneId, ids.sceneNumber, ids.groupId);
    }
});

Ext.reg('dssscenespanel', dSS.ScenesPanel);

