//= require <dSS/GetTextHelper>
//= require <dSS/ZoneView>
Ext.namespace('dSS');

dSS.ZonePanel = Ext.extend(Ext.Panel, {
    initComponent: function() {
        Ext.apply(this, {
            title: _("Rooms"),
            layout: 'border',
            items: [
            {
                xtype: 'dsszoneview',
                ref: 'zoneView',
                region: 'center'
            }]
            //tbar: ['->', createZoneAction, reloadAction]
        });

        this.tbar = this.buildTopToolbar();

        this.zoneSortMode = 0;

        dSS.ZonePanel.superclass.initComponent.apply(this, arguments);
    },

    buildTopToolbar: function() {
        return [
        {
            //text: 'New Zone',
            iconCls: 'newZoneAction',
            handler: this.createNewZone,
            tooltip: _("Create new room"),
            scope: this
        },
        {
            //text: 'Edit Room
            iconCls: 'editZoneAction',
            handler: this.editZone,
            tooltip: _("Rename room"),
            scope: this
        },

        '->', {
            iconCls: 'sortZoneAction',
            handler: this.sortZonesCycle,
            tooltip: _("Click to cycle between sorting by id (ascending/descending) and room name (ascending/descending)"),
            scope: this
        },

        {
            cls: 'zonePanelMenu',
            menu: [
            {
                text: _("Reload"),
                handler: this.reload,
                scope: this
            },
            {
                text: _("Rescan"),
                handler: this.rescan,
                scope: this
            }]
        }];
    },
    rescan: function() {
        this.ownerCt.rescan();
    },
    editZone: function() {
        if (this.zoneView.getSelectionCount() !== 1) {
            return;
        } else {
            var record = this.zoneView.getSelectedRecords()[0];
            Ext.Msg.prompt(_("Rename room"), _("New name for room"), function(btn, text) {
                if (text !== record.get('name')) {
                    Ext.Ajax.request({
                        url: '/json/zone/setName',
                        disableCaching: true,
                        method: "GET",
                        params: {
                            id: record.get('id'),
                            newName: text
                        },
                        success: function(result, request) {
                            try {
                                var jsonData = Ext.util.JSON.decode(result.responseText);
                                if (jsonData.ok) {
                                    record.set('name', text);
                                    record.commit();
                                } else {
                                    Ext.MessageBox.alert(_("Error"), _("Could not rename room"));
                                }
                            }
                            catch(err) {
                                Ext.MessageBox.alert(_("Error"), _("Could not rename room"));
                            }
                        },
                        failure: function(result, request) {
                            Ext.MessageBox.alert(_("Error"), _("Could not rename room"));
                        }
                    });
                }
            },
            this, false, record.get('name'));
        }
    },

    createNewZone: function() {
        var zoneStore = this.zoneView.getStore();
        Ext.Msg.prompt(_("Create new room"), _("Name for new room"), function(btn, text) {
            if (btn === 'ok') {
                for (var i = 0; i <= zoneStore.data.length; i++) {
                    if (zoneStore.findExact('id', i) === - 1) {

                        Ext.Ajax.request({
                            url: '/json/structure/addZone',
                            disableCaching: true,
                            method: "GET",
                            params: {
                                zoneID: i
                            },
                            success: function(result, request) {
                                try {
                                    var jsonData = Ext.util.JSON.decode(result.responseText);
                                    if (jsonData.ok) {

                                        Ext.Ajax.request({
                                            url: '/json/zone/setName',
                                            disableCaching: true,
                                            method: "GET",
                                            params: {
                                                id: i,
                                                newName: text
                                            },
                                            success: function(result, request) {
                                                try {
                                                    var jsonData = Ext.util.JSON.decode(result.responseText);
                                                    if (jsonData.ok) {
                                                        zoneStore.loadData({
                                                            zones: [
                                                            {
                                                                id: i,
                                                                name: text
                                                            }]
                                                        },
                                                        true);
                                                    } else {
                                                        Ext.MessageBox.alert(_("Error"), _("Could not create zone") + ": " + json.message);
                                                    }
                                                }
                                                catch(err) {
                                                    Ext.MessageBox.alert(_("Error"), _("Could not create zone") + ": " + err);
                                                }
                                            },
                                            failure: function(result, request) {
                                                Ext.MessageBox.alert(_("Error"), _("Could not create zone"));
                                            }
                                        });

                                    }
                                }
                                catch(err) {
                                    Ext.MessageBox.alert(_("Error"), _("Could not create zone"));
                                }
                            },
                            failure: function(result, request) {
                                Ext.MessageBox.alert(_("Error"), _("Could not create zone"));
                            }
                        });
                        return;
                    }
                }
            }
        });
    },
    reload: function() {
        this.ownerCt.loadData();
    },
    sortZonesCycle: function() {
        var zstore = dSS.data.Loader.getZoneStore();
        switch (this.zoneSortMode) {
        case 0:
            zstore.sort('id', 'ASC');
            this.zoneSortMode++;
            break;
        case 1:
            zstore.sort('id', 'DESC');
            this.zoneSortMode++;
            break;
        case 2:
            zstore.sort('name', 'ASC');
            this.zoneSortMode++;
            break;
        case 3:
            zstore.sort('name', 'DESC');
            this.zoneSortMode = 0;
            break;
        default:
            this.zoneSortMode = 0;
            break;
        }
    }
});

Ext.reg('dsszonepanel', dSS.ZonePanel);

