Ext.namespace('dSS', 'dSS.data');

dSS.data.DeviceStore = Ext.extend(Ext.data.Store, {
    constructor: function(config) {
        // create a record constructor for device records
        var deviceRecord = Ext.data.Record.create([
        {
            name: "name"
        },
        {
            name: "dsid",
            mapping: "id"
        },
        {
            name: "blink",
            defaultValue: false
        },
        {
            name: "meterDSID"
        },
        {
            name: "modulatorName"
        },
        {
            name: "zoneId"
        },
        {
            name: "isPresent"
        },
        {
            name: "firstSeen"
        },
        {
            name: "lastDiscovered"
        },
        {
            name: "locked"
        },
        {
            name: "functionID"
        },
        {
            name: "comment"
        },
        {
            name: "productRevision"
        },
        {
            name: "productID"
        },
        {
            name: "HW"
        }]);

        // a json reader to read the device data
        var deviceReader = new Ext.data.JsonReader({
            root: "devices"
        },
        deviceRecord);

        Ext.apply(
        this, {
            reader: deviceReader,
            sortInfo: {
                field: 'firstSeen',
                direction: 'DESC' // or 'DESC' (case sensitive for local sorting)
            }
        });

        dSS.data.DeviceStore.superclass.constructor.call(this, arguments);

        this.on('load', function(store, records, options) {

            Ext.each(records, function(record) {
                var rev = record.get('productRevision');
                var major = rev >> 8;
                var mid = rev & 0x00ff;

                var hwString = dSS.util.deviceStringFromProduct(
                                                    record.get('functionID'),
                                                    record.get('productID'));

                record.set('HW', hwString + ", " + dSS.util.dec2hex(major) +
                           "." + dSS.util.dec2hex(mid));
                record.commit();
             });

            this.loadComments(records); 
        },
        this);
    },
    loadComments: function(records) {
        if (records.length === 0) {
            return;
        }

        var record = records.shift();

        Ext.Ajax.request({
            url: '/json/property/getString',
            disableCaching: true,
            method: 'GET',
            scope: this,
            params: {
                path : '/apartment/zones/zone0/' + 
                        record.get('dsid') + '/comment'
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        record.set('comment', jsonData.result.value);
                        record.commit();
                    }
                    this.loadComments(records);
                } catch (err) {
                    Ext.MessageBox.alert(_("Error"), _("Could not get comment from dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(_("Error"), _("Failure when requesting comment from dSS"));
            }
        });
    }
});

