//= require <dSS/data/ModulatorStore>
//= require <dSS/data/DeviceStore>
//= require <dSS/data/ZoneStore>
Ext.namespace('dSS', 'dSS.data');

dSS.data.LoaderImpl = Ext.extend(Ext.util.Observable, {

    constructor: function() {
        this.modulatorStore = new dSS.data.ModulatorStore({
            url: '/json/apartment/getCircuits'
        });
        this.deviceStore = new dSS.data.DeviceStore();
        this.zoneStore = new dSS.data.ZoneStore();

        this.modulatorStore.on('loadcomplete',
                               this.loadStructure.createDelegate(this));

    },

    load: function() {
        this.modulatorStore.load();
    },

    getModulatorStore: function() {
        return this.modulatorStore;
    },

    getDeviceStore: function() {
        return this.deviceStore;
    },

    getZoneStore: function() {
        return this.zoneStore;
    },

    loadStructure: function() {
        Ext.Ajax.request({
            url: '/json/apartment/getStructure',
            disableCaching: true,
            timeout: 300000,
            method: "GET",
            scope: this,
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    this.processStructure(jsonData.result);
                }
                catch(err) {
                    Ext.MessageBox.alert(_("Error"), _("Could not load appartment structure") + ": \"" + err + "\"");
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(_("Error"), _("Could not load") + ": \"" + request.url + "\"");
            }
        });
    },

    processStructure: function(structure) {
        var devices = [],
        zones = [];
        var zoneStore = this.zoneStore;
        var modulatorStore = this.modulatorStore;
        var deviceStore = this.deviceStore;
        Ext.each(structure.apartment.zones, function(zone) {
            if (zone.id === 0) {
                Ext.each(zone.devices, function(device) {
                    device.firstSeen = Date.parseDate(device.firstSeen + "+00:00", "c");
                    device.lastDiscovered = Date.parseDate(device.lastDiscovered + "+00:00", "c");
                    devices.push(device);
                });
            }
            zones.push({
                id: zone.id,
                name: zone.name ? zone.name: _("Zone"),
                primary: ('firstZoneOnDSMeter' in zone) ? true: false
            });
            Ext.each(zone.devices, function(device) {
                for (var i = 0; i < devices.length; i++) {
                    if (devices[i].id === device.id) {
                        devices[i].zoneId = zone.id;
                    }
                }
            });
            zoneStore.loadData({
                zones: zones
            });
        });
        Ext.each(devices, function(device) {
            var index = modulatorStore.find('dsid',
                        new RegExp("^" + device.meterDSID + "$"));
            if (index > - 1) {
                var modulator = modulatorStore.getAt(index);
                device.modulatorName = modulator.get("name");
            }
        });
        deviceStore.loadData({
            devices: devices
        });
    }

});

