//= require <dSS/GetTextHelper>
//= require <dSS/data/DeviceStore>
//= require <dSS/data/Loader>
//= require <dSS/util/GridToSVWindow>
//= require <dSS/util/ExtendedPingForm>
//= require <dSS/util/ExtendedPingWindow>
//= require <dSS/util/DeviceStatisticsForm>
//= require <dSS/util/DeviceStatisticsWindow>
//= require <dSS/util/Util>
//= requie  <dSS/config/Config>
//= require <dSS/scene/SceneWindowShadow>
//= require <dSS/scene/SceneWindowLight>
Ext.namespace('dSS', 'dSS.grid');

dSS.grid.DevicePanel = Ext.extend(Ext.grid.GridPanel, {
    initComponent: function() {

        var blinkRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            if (record.get('isPresent') === false) {
                metaData.css = 'isDisabled';
            }
            else {
                if (record.get('blink') === true) {
                    metaData.css = 'blinkActive';
                } else {
                    metaData.css = 'blink';
                }
            }
            return String.format('<img class="padding-img" src="{0}"/>', Ext.BLANK_IMAGE_URL);
        };

        var commentRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            metaData.css = 'commentIcon';

            var comment = record.get('comment');

            if ((comment !== undefined) && (comment.length > 0)) {
                return String.format('<div qtip="{0}">&nbsp;&nbsp;&nbsp;</div>',
                        comment);
            } else {
                return '<div>&nbsp;&nbsp;&nbsp;</div>';
            }

        };

        var lockRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            if (record.get('isPresent') === false) {
                metaData.css = 'lockDisabled';
            }
            else {
                if (record.get('locked')) {
                    metaData.css = 'isLocked';
                } else {
                    metaData.css = 'isUnlocked';
                }
            }
            return String.format('<img class="padding-img" src="{0}"/>', Ext.BLANK_IMAGE_URL);
        };

        var devTypeRenderer = function(value, metaData, record, rowIndex, colIndex, store) {

            var present = record.get('isPresent');

            var fid = dSS.util.parseFunctionID(record.get('functionID'));

            var retval = String.format('<img class="padding-img" src="{0}"/>', Ext.BLANK_IMAGE_URL);

            switch (fid[0]) {
            case dSS.config.groups.LIGHT:
                if (present === true) {
                    metaData.css = 'functionLight';
                } else {
                    metaData.css = 'functionLightDisabled';
                }
                break;
            case dSS.config.groups.SHADOW:
                if (present === true) {
                    metaData.css = 'functionShadow';
                } else {
                    metaData.css = 'functionShadowDisabled';
                }
                break;
            case dSS.config.groups.CLIMATE:
                metaData.css = 'functionClimate';
                break;
            case dSS.config.groups.AUDIO:
                metaData.css = 'functionAudio';
                break;
            case dSS.config.groups.VIDEO:
                metaData.css = 'functionVideo';
                break;
            case dSS.config.groups.SECURITY:
                metaData.css = 'functionSecurity';
                break;
            case dSS.config.groups.ACCESS:
                metaData.css = 'functionAccess';
                break;
            case dSS.config.groups.JOKER:
                metaData.css = 'functionJoker';
                break;
            default:
                retval = "";
                break;
            }

            return retval;
        };

        var deviceCols = [
        {
            header: "",
            width: 17,
            resizable: false,
            sortable: false,
            hideable: false,
            menuDisabled: true,
            fixed: true,
            dataIndex: 'functionID',
            renderer: devTypeRenderer,
            id: 'functionID'
        },
        {
            header: _("blink"),
            width: 25,
            resizable: false,
            sortable: false,
            dataIndex: 'blink',
            renderer: blinkRenderer,
            id: 'blink'
        },
        {
            header: "dsid",
            width: 150,
            sortable: true,
            dataIndex: 'dsid',
            id: 'dsid'
        },
        {
            header: _("name"),
            width: 150,
            sortable: true,
            dataIndex: 'name',
            id: 'name'
        },
        {
            header: _("meter name"),
            width: 100,
            sortable: true,
            dataIndex: 'modulatorName',
            id: 'modulatorName'
        },
        {
            header: _("meter dsid"),
            width: 150,
            sortable: true,
            dataIndex: 'meterDSID',
            id: 'meterDSID'
        },
        {
            header: _("zone id"),
            width: 50,
            sortable: true,
            dataIndex: 'zoneId',
            id: 'zoneId'
        },
        {
            header: _("first seen"),
            width: 150,
            sortable: true,
            dataIndex: 'firstSeen',
            xtype: 'datecolumn',
            format: 'Y-m-d\\&\\n\\b\\s\\p\\;H:m:i',
            id: 'firstSeen'
        },
        {
            header: _("last discovered"),
            width: 150,
            sortable: true,
            dataIndex: 'lastDiscovered',
            xtype: 'datecolumn',
            format: 'Y-m-d\\&\\n\\b\\s\\p\\;H:m:i',
            id: 'lastDiscovered'
        },
        {
            header: _("Comment"),
            width: 20,
            resizable: false,
            sortable: false,
            dataIndex: 'comment',
            id: 'comment',
            renderer: commentRenderer
        },
        {
            header: _("locked"),
            width: 20,
            sortable: true,
            dataIndex: 'locked',
            id: 'lock',
            renderer: lockRenderer
        },
        {
            header: _("HW Info"),
            width: 80,
            sortable: true,
            id: 'HW',
            dataIndex: 'HW'
        }];

        Ext.apply(this, {
            store: dSS.data.Loader.getDeviceStore(),
            columns: deviceCols,
            ddGroup: "zoneDeviceDD",
            enableDragDrop: true,
            stripeRows: true,
            forceFit: true,
            title: gt.ngettext("Device", 2),
            viewConfig: {
                autoFill: true,
                getRowClass: function(record, index) {
                    var c = record.get('isPresent');
                    if (c === false) {
                        return 'nonPresentDevice';
                    }
                    return '';
                }
            }
        });

        this.tbar = this.buildTopToolbar();

        dSS.grid.DevicePanel.superclass.initComponent.apply(this, arguments);

        // Here you can add functionality that requires the object to
        // exist, like event handling.
        this.on('rowcontextmenu', function(grid, rowIndex, event) {
            event.preventDefault();
            event.stopEvent();
            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }

            var sel = this.getSelectionModel();
            if (!sel.isSelected(rowIndex)) {
                sel.selectRow(rowIndex, false);
            }

            var showRemoveMenuItem = false;
            var presentInSelection = false;
            var showPingMenuItem = false;

            if (sel.getSelected().get("isPresent") === true) {
                showPingMenuItem = true;
            }

            var records = sel.getSelections();

            for (var i = 0; i < records.length; i++) {
                var record = records[i];

                // if at least one record can be removed - set the flag
                if (record.get('isPresent') === false) {
                    showRemoveMenuItem = true;
                } else {
                    presentInSelection = true;
                }

                if ((showRemoveMenuItem === true) && (presentInSelection === true)) {
                    break;
                }
            }

            var menuItem = new Ext.menu.Item({
                text: _("Rename device"),
                icon: 'images/page_white_edit.png',
                handler: this.editDevice.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            if (showPingMenuItem === true) {
                var menuItem = new Ext.menu.Item({
                    text: _("Send diagnostic ping"),
                    icon: 'images/application_osx_terminal.png',
                    handler: this.pingDevice.createDelegate(this, rowIndex, true)
                });
                this.contextMenu.add(menuItem);
            }

            if (dSS.config.features.extendedPing === true) {
                var subMenu = new Ext.menu.Menu({});

                if (presentInSelection === true) {
                    var menuItem = new Ext.menu.Item({
                        text: _("Extended diagnostic ping"),
                        icon: 'images/application_osx_terminal.png',
                        handler: this.extendedPing.createDelegate(this, rowIndex, true)
                    });
                    subMenu.add(menuItem);
                }

                var menuItem = new Ext.menu.Item({
                    text: _("Extended diagnostic ping results"),
                    icon: 'images/application_osx_terminal.png',
                    handler: this.showExtendedPingResults.createDelegate(this, rowIndex, true)
                });
                subMenu.add(menuItem);

                var menuItem = new Ext.menu.Item({
                    text: _("Abort ping session"),
                    icon: 'images/application_osx_terminal.png',
                    handler: this.abortRound.createDelegate(this, ["extendedPing", _("Ping")], true)
                });
                subMenu.add(menuItem);

                var menuItem = new Ext.menu.Item({
                    text: _("Extended ping"),
                    icon: 'images/application_osx_terminal.png',
                    menu: subMenu
                });
                this.contextMenu.add(menuItem);
            }

            if (showRemoveMenuItem === true) {
                var menuItem = new Ext.menu.Item({
                    text: _("Remove device"),
                    icon: 'images/delete.png',
                    handler: this.removeDevice.createDelegate(this, rowIndex, true)
                });
                this.contextMenu.add(menuItem);
            }

            if (dSS.config.features.deviceStatistics === true) {
                var subMenu = new Ext.menu.Menu({});

                if (presentInSelection === true) {
                    var menuItem = new Ext.menu.Item({
                        text: _("Statistics"),
                        icon: 'images/application_osx_terminal.png',
                        handler: this.deviceStatistics.createDelegate(this, rowIndex, true)
                    });
                    subMenu.add(menuItem);
                }

                var menuItem = new Ext.menu.Item({
                    text: _("Abort statistics session"),
                    icon: 'images/application_osx_terminal.png',
                    handler: this.abortRound.createDelegate(this, ["deviceStatistics", _("Device statistics")], true)
                });
                subMenu.add(menuItem);

                var menuItem = new Ext.menu.Item({
                    text: _("Device statistics"),
                    icon: 'images/application_osx_terminal.png',
                    menu: subMenu
                });
                this.contextMenu.add(menuItem);
            }

            if (presentInSelection === true) {
                var menuItem = new Ext.menu.Item({
                    text: _("Blink"),
                    icon: 'images/blink.png',
                    handler: this.blinkDevice.createDelegate(this, rowIndex, true)
                });
                this.contextMenu.add(menuItem);
            }

            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
        this);

        this.on('cellclick', function(grid, rowIndex, columnIndex, event) {

            var dataIndex = grid.getColumnModel().getDataIndex(columnIndex);

            var record = this.getStore().getAt(rowIndex);

            if (dataIndex === 'comment') {
                this.editComment(record);
                return;
            }

            if (record.get('isPresent') === false) {
                return;
            }

            if (dataIndex === 'blink') {
                this.sendBlinkRequest(record);
            } else if (dataIndex === 'locked') {
                Ext.Ajax.request({
                    url: record.get('locked') ? '/json/device/unlock': '/json/device/lock',
                    disableCaching: true,
                    method: "GET",
                    params: {
                        dsid: record.get('dsid')
                    },
                    success: function(result, request) {
                        try {
                            var jsonData = Ext.util.JSON.decode(result.responseText);
                            if (jsonData.ok) {
                                record.set('locked', record.get('locked') ? false: true);
                                record.commit();
                            }
                        } catch(err) {
                            Ext.MessageBox.alert('Error', err);
                        }
                    }
                });
            } else if (dataIndex === 'functionID') {
                var fid = dSS.util.parseFunctionID(record.get('functionID'));
                switch (fid[0]) {
                case 1:
                    var win = new dSS.scene.SceneWindowLight({
                        dsid: record.get('dsid'),
                        name: record.get('name'),
                        maskElement: this.findParentByType(Ext.Viewport).el
                    });
                    win.show(this);
                    break;
                case 2:
                    var win = new dSS.scene.SceneWindowShadow({
                        dsid: record.get('dsid'),
                        name: record.get('name'),
                        maskElement: this.findParentByType(Ext.Viewport).el
                    });
                    win.show(this);
                    break;
                default:
                    break;
                }
            } else if ((dataIndex === 'dsid') || (dataIndex === 'name')) {
                this.editDevice();
            }
        },
        this);
    },

    buildTopToolbar: function() {
        return ['->', {
            iconCls: 'copyToClipboardToolbarIcon',
            handler: this.showCopyWindow,
            tooltip: _("Show in copy to cliboard view"),
            scope: this
        },
        {
            iconCls: 'editDeviceToolbarIcon',
            handler: this.editDevice,
            tooltip: _("Rename device"),
            scope: this
        },
        {
            iconCls: 'pingDeviceToolbarIcon',
            handler: this.pingDevice,
            tooltip: _("Send diagnostic ping"),
            scope: this
        },
        {
            iconCls: 'removeDeviceToolbarIcon',
            handler: this.removeDevice,
            tooltip: _("Remove device"),
            scope: this
        }];
    },

    editDevice: function(item, event, rowIndex) {
        var record;
        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(_("No device selected"), _("Please select a device from the list!"));
            return;
        }

        if (sel.getSelections().length > 1) {
            Ext.Msg.alert(_("Multiple devices selected"), _("It is not possible to edit multiple devices simultaneously, please select only one device!"));
            return;
        }

        record = sel.getSelected();

        Ext.Msg.prompt(_("Rename device"), _("Name") + ": ", function(btn, text) {
            if (text !== record.get('name')) {
                Ext.Ajax.request({
                    url: '/json/device/setName',
                    disableCaching: true,
                    method: "GET",
                    params: {
                        dsid: record.get('dsid'),
                        newName: text
                    },
                    success: function(result, request) {
                        try {
                            var jsonData = Ext.util.JSON.decode(result.responseText);
                            if (jsonData.ok) {
                                record.set('name', text);
                                record.commit();
                            } else {
                                Ext.MessageBox.alert(_("Error"), _("Could not rename device"));
                            }
                        }
                        catch(err) {
                            Ext.MessageBox.alert(_("Error"), _("Could not rename device"));
                        }
                    },
                    failure: function(result, request) {
                        Ext.MessageBox.alert(_("Error"), _("Could not rename device"));
                    }
                });
            }
        },
        this, false, record.get('name'));
    },

    pingDevice: function(item, event, rowIndex) {
        var record;
        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(_("No device selected"), _("Please select a device from the list!"));
            return;
        }

        if (sel.getSelections().length > 1) {
            Ext.Msg.alert(_("Multiple devices selected"), _("It is not possible to ping multiple devices simultaneously, please select only one device or use extended ping!"));
            return;
        }

        record = sel.getSelected();

        if (record.get('isPresent') === false) {
            Ext.Msg.alert(_("Device not present"), _("The selected device was not found and can not be pinged!"));
            return;
        }

        Ext.Ajax.request({
            url: '/json/debug/pingDevice',
            disableCaching: true,
            method: "GET",
            params: {
                dsid: record.get('dsid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        var title = _("Ping") + " " + record.get('dsid');
                        var message = _("SC") + ": " + jsonData.result.qualityHK + "<br/>" + _("RC") + ": " + jsonData.result.qualityRK;
                        Ext.MessageBox.show({
                            title: title,
                            msg: message,
                            buttons: Ext.Msg.OK,
                            minWidth: 300
                        });
                    } else {
                        Ext.MessageBox.alert(_("Ping error"), jsonData.message);
                    }
                }
                catch(err) {
                    Ext.MessageBox.alert(_("Error"), _("Could send ping command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(_("Error"), _("Could send ping command to dSS"));
            }
        });
    },

    extendedPing: function(item, event, rowIndex) {
        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(_("No device selected"), _("Please select a device from the list!"));
            return;
        }

        records = sel.getSelections();

        var dsids = "";
        var deviceArray = [];
        for (var i = 0; i < records.length; i++) {
            if (records[i].get("isPresent") === false) {
                continue;
            }
            var device = {};
            device.dsid = records[i].get("dsid");
            device.name = records[i].get("name");
            deviceArray.push(device);
            dsids += device.dsid;
            if (i < (records.length - 1)) {
                dsids += ",";
            }
        }

        if (deviceArray.length === 0) {
            Ext.Msg.alert(_("No devices present"), _("Your selection contained only non present devices which can not be pinged!"));
            return;
        }

        var deviceRecord = Ext.data.Record.create([
        {
            name: "dsid",
            mapping: "dsid"
        },
        {
            name: "name",
            mapping: "name"
        }]);

        var pingStore = new Ext.data.ArrayStore({
            autoDestroy: true,
            storeId: "pingStore",
            idIndex: 0,
            fields: deviceRecord,
            data: deviceArray
        });

        var now = new Date();
        var h_now = now.getHours();
        var m_now = now.getMinutes();

        m_now += 5;
        if (m_now >= 60) {
            h_now++;
            m_now = m_now - 60;
        }

        if (h_now >= 24) { 
            h_now = 0;
        }

        var sep = ":";

        if (m_now < 10) {
            sep = sep + "0";
        }

        var pingForm = new dSS.util.ExtendedPingForm({
            pingStore: pingStore,
            sep: sep,
            h_now: h_now,
            m_now: m_now
        });

        var win = new dSS.util.ExtendedPingWindow({
            form: pingForm,
            dsids: dsids,
            maskElement: this.findParentByType(Ext.Viewport).el
        });

        win.show(this);
    },

    removeDevice: function(item, event, rowIndex) {
        var records;

        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.MessageBox.alert(_("No device selected"), _("Please select a device from the list!"));
            return;
        }
        records = sel.getSelections();

        var skippedPresent = false;
        var notPresent = [];

        for (var i = 0; i < records.length; i++) {
            var record = records[i];

            // do not issue the ajax request if we know that it will fail
            if (record.get('isPresent') === true) {
                skippedPresent = true;
                continue;
            }

            notPresent.push(record);
        }

        if (notPresent.length > 0) {
            this.deleteDevice(notPresent);
        } else {
            if (skippedPresent === true) {
                Ext.MessageBox.alert(_("Warning"), _("Present devices can not be removed!"));
            }
        }
    },

    deleteDevice: function(records) {
        var store = this.getStore();
        if (records.length < 1) {
            this.el.unmask();
            return;
        }

        var record = records.shift();

        this.el.mask(_("Deleting device") + ": " + record.get("dsid"));
        Ext.Ajax.request({
            url: '/json/structure/removeDevice',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                deviceID: record.get('dsid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        store.remove(record);
                        this.deleteDevice(records);
                    } else {
                        this.el.unmask();
                        Ext.MessageBox.alert(_("Could not remove"), jsonData.message);
                    }
                }
                catch(err) {
                    this.el.unmask();
                    Ext.MessageBox.alert(_("Error"), _("Could send remove device command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                this.el.unmask();
                Ext.MessageBox.alert(_("Error"), _("Could send remove device command to dSS"));
            }
        });
    },

    sendBlinkRequest: function(record) {
        record.set('blink', true);
        var unblink = function(record) {
            record.set('blink', false);
        };

        Ext.Ajax.request({
            url: '/json/device/blink',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                dsid: record.get('dsid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (!jsonData.ok) {
                        Ext.MessageBox.alert(_("Could not blink"), jsonData.message);
                    }
                } catch(err) {
                    Ext.MessageBox.alert(_("Error"), _("Could not send blink device command to dSS") + ": " + err);
                }
                unblink.defer(2000, this, [record]);
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(_("Error"), _("Could not send blink device command to dSS"));
                unblink.defer(2000, this, [record]);
            }
        });
    },

    blinkDevice: function(item, event, rowIndex) {
        var records;

        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.MessageBox.alert(_("No device selected"), _("Please select a device from the list!"));
            return;
        }
        records = sel.getSelections();
        for (var i = 0; i < records.length; i++) {
            if (records[i].get('isPresent') === false) {
                continue;
            }

            this.sendBlinkRequest(records[i]);

        }
    },

    showCopyWindow: function() {
        var tsvwindow = new dSS.util.GridToSVWindow({
            ignore: [ 'blink', 'functionID', 'lock' ],
            gridPanel: this,
            windowWidth: 640,
            windowHeight: 480,
            separator: "\t"
        });
        tsvwindow.show(this);
    },

    abortRound: function(p0, p1, property, text) {
        Ext.Ajax.request({
            url: '/json/property/getInteger',
            disableCaching: true,
            method: 'GET',
            scope: this,
            params: {
                path: '/system/js/settings/' + property + '/session'
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        var old_session = jsonData.result.value;
                        var new_session = old_session + 1;
                        Ext.Ajax.request({
                            url: '/json/property/setInteger',
                            disableCaching: true,
                            method: 'GET',
                            scope: this,
                            params: {
                                path: '/system/js/settings/' + property + '/session',
                                value: new_session
                            },
                            success: function(result, request) {
                                try {
                                    var jsonData = Ext.util.JSON.decode(result.responseText);
                                    if (jsonData.ok) {
                                        Ext.MessageBox.alert(_("Information"), text + " " + _("session") + " #" + old_session + " " + _("aborted") + ".");
                                    } else {
                                        Ext.MessageBox.alert(_("Could not set session value"), jsonData.message);
                                    }
                                } catch(err) {
                                    Ext.MessageBox.alert(_("Error"), _("Could send command to dSS") + ": " + err);
                                }
                            },
                            failure: function(result, request) {
                                Ext.MessageBox.alert(_("Error"), _("Could send command to dSS"));
                            }
                        });
                    } else {
                        Ext.MessageBox.alert(_("Could not get session value"), jsonData.message);
                    }
                } catch(err) {
                    Ext.MessageBox.alert(_("Error"), _("Could send command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(_("Error"), _("Could not send command to dSS"));
            }
        });
    },

    showExtendedPingResults: function() {
        Ext.Ajax.request({
            url: '/json/event/subscribe',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                subscriptionID: 333,
                name: "ping_result"
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        var pingResult = new dSS.util.PingEventWindow({
                            windowWidth: 640,
                            windowHeight: 480,
                            maskElement: this.findParentByType(Ext.Viewport).el
                        });
                        pingResult.show(this);
                    } else {
                        Ext.MessageBox.alert(_("Ping subscription error"), jsonData.message);
                    }
                } catch(err) {
                    Ext.MessageBox.alert(_("Error"), _("Could send subscribe command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(_("Error"), _("Could send subscribe command to dSS"));
            }
        });
    },

    deviceStatistics: function(item, event, rowIndex) {
        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(_("No device selected"), _("Please select a device from the list!"));
            return;
        }

        records = sel.getSelections();

        var dsids = "";
        var deviceArray = [];
        for (var i = 0; i < records.length; i++) {
            if (records[i].get("isPresent") === false) {
                continue;
            }
            var device = {};
            device.dsid = records[i].get("dsid");
            device.name = records[i].get("name");
            deviceArray.push(device);
            dsids += device.dsid;
            if (i < records.length - 1) {
                dsids += ",";
            }
        }

        if (deviceArray.length === 0) {
            Ext.Msg.alert(_("No devices present"), _("Your selection contained only non present devices!"));
            return;
        }

        var deviceRecord = Ext.data.Record.create([
        {
            name: "dsid",
            mapping: "dsid"
        },
        {
            name: "name",
            mapping: "name"
        }]);

        var statsStore = new Ext.data.ArrayStore({
            autoDestroy: true,
            storeId: "statsStore",
            idIndex: 0,
            fields: deviceRecord,
            data: deviceArray
        });

        var now = new Date();
        var h_now = now.getHours();
        var m_now = now.getMinutes();

        m_now += 5;
        if (m_now >= 60) {
            h_now++;
            m_now = m_now - 60;
        }

        if (h_now >= 24) { 
            h_now = 0;
        }

        var sep = ":";

        if (m_now < 10) {
            sep = sep + "0";
        }

        var statsForm = new dSS.util.DeviceStatisticsForm({
            statisticsStore: statsStore,
            sep: sep,
            h_now: h_now,
            m_now: m_now
        });

        var win = new dSS.util.DeviceStatisticsWindow({
            form: statsForm,
            dsids: dsids,
            maskElement: this.findParentByType(Ext.Viewport).el
        });

        win.show(this);
    },
    editComment: function(record) {
        Ext.Msg.prompt(
            _("Edit comment"), 
            _("Comment") + ": ", 
            function(btn, text) {
            if (text !== record.get('comment')) {
                Ext.Ajax.request({
                    url: '/json/property/setString',
                    disableCaching: true,
                    method: "GET",
                    params: {
                        path:   '/apartment/zones/zone0/' +
                                record.get('dsid') + '/comment',
                        value: text
                    },
                    success: function(result, request) {
                        try {
                            var jsonData = Ext.util.JSON.decode(
                                                        result.responseText);
                            if (jsonData.ok) {
                                Ext.Ajax.request({
                                    url: '/json/property/setFlag',
                                    disableCaching: true,
                                    method: "GET",
                                    params: {
                                        flag: 'ARCHIVE',
                                        path:   '/apartment/zones/zone0/' +
                                                record.get('dsid') + '/comment',
                                        value: true
                                    },
                                    success: function(result, request) {
                                        try {
                                            var jsonData = Ext.util.JSON.decode(
                                                result.responseText);
                                            if (jsonData.ok) {
                                                record.set('comment', text);
                                                record.commit();
                                            }
                                        } catch(err) {
                                            Ext.MessageBox.alert(_("Error"), 
                                            _("Could not save comment on dSS") +
                                              ": " + err);
                                        }
                                    },
                                    failure: function(result, request) {
                                        Ext.MessageBox.alert(_("Error"), 
                                            _("Could not save comment on dSS"));
                                    }
                                });
                            } else {
                                Ext.MessageBox.alert(_("Error"), 
                                                    _("Could not add comment"));
                            }
                        }
                        catch(err) {
                            Ext.MessageBox.alert(_("Error"), 
                                     _("Could not add comment") + ": " + err);
                        }
                    },
                    failure: function(result, request) {
                        Ext.MessageBox.alert(_("Error"), 
                                _("Could not send command to dSS"));
                    }
                });
            }
        },
        this, false, record.get('comment'));
    }
});

Ext.reg('dssdevicepanel', dSS.grid.DevicePanel);

