//= require <dSS/GetTextHelper>
//= require <dSS/data/Loader>
//= require <dSS/util/DSMStatisticsWindow>
Ext.namespace('dSS', 'dSS.grid', 'dSS.data', 'dSS.config');

dSS.grid.ModulatorPanel = Ext.extend(Ext.grid.GridPanel, {
    initComponent: function() {

        var scanStatus = function(value, metaData, record, rowIndex, colIndex, store) {

            var v = record.get('isValid');
            var p = record.get('isPresent');

            if ((p === true) && (v === false)) {
                metaData.attr = 'ext:qtip="' + _("Reading circuit data") +
                "..." + '"';
                if (colIndex === 0) {
                    metaData.css = 'loading-indicator';
                }
            }

            if (colIndex === 0) {
                return '';
            }

            return value;
        };

        var modulatorColumns = [
        {
            id: 'isValid',
            width: 8,
            resizable: false,
            menuDisabled: true,
            sortable: false,
            dataIndex: 'isValid',
            renderer: scanStatus
        },
        {
            id: 'dsid',
            header: _("dsid"),
            width: 190,
            sortable: true,
            dataIndex: 'dsid',
            renderer: scanStatus
        },
        {
            id: 'name',
            header: _("meter name"),
            width: 150,
            sortable: true,
            dataIndex: 'name',
            renderer: scanStatus
        },
        {
            id: 'consumption',
            header: _("consumption (mW)"),
            width: 130,
            sortable: true,
            dataIndex: 'consumption'
        },
        {
            id: 'measureDate',
            header: _("measured"),
            width: 150,
            sortable: true,
            dataIndex: 'measureDate'
        },
        {
            id: 'hwVersion',
            header: _("HW version"),
            width: 30,
            sortable: true,
            dataIndex: 'hwVersion'
        },
        {
            id: 'armSwVersion',
            header: _("ARM software version"),
            width: 30,
            sortable: true,
            dataIndex: 'armSwVersion'
        },
        {
            id: 'dspSwVersion',
            header: _("DSP software version"),
            width: 30,
            sortable: true,
            dataIndex: 'dspSwVersion'
        },
        {
            id: 'apiVersion',
            header: _("API version"),
            width: 20,
            sortable: true,
            dataIndex: 'apiVersion'
        }];

        var rowSelect = new Ext.grid.RowSelectionModel({
            singleSelect: false
        });

        this.heightConfigured = false;

        Ext.apply(this, {
            store: dSS.data.Loader.getModulatorStore(),
            columns: modulatorColumns,
            enableDragDrop: false,
            stripeRows: true,
            forceFit: true,
            title: _("Meters"),
            selModel: rowSelect,
            viewConfig: {
                autoFill: true,
                getRowClass: function(record, index) {
                    var c = record.get('isPresent');
                    if (c === false) {
                        return 'nonPresentDevice';
                    }

                    var v = record.get('isValid');
                    if (v === false) {
                        return 'scanInProgress';
                    }

                    return '';
                }
            }
        });

        this.tbar = this.buildTopToolbar();

        dSS.grid.ModulatorPanel.superclass.initComponent.apply(this, arguments);

        this.store.on('load', function(store, records, options) {
            var scanning = false;
            Ext.each(records, function(record) {
                var p = record.get('isPresent');
                var v = record.get('isValid');

                if ((p === true) && (v === false)) {
                    scanning = true;
                    return false;
                }
                return true;
            });

            if (scanning === true) {
                this.reload.defer(15000, this);
            }
        });

        this.on('afterrender', function() {
            if (this.heightConfigured === false) {
                this.heightConfigured = true;
                this.setHeight(this.findParentByType('dsmoverviewpanel').getInnerHeight() / 2);
            }

        });

        this.on('viewready', function() {
            this.setInitialSelection();
        });

        this.on('rowcontextmenu', function(grid, rowIndex, event) {
            event.preventDefault();
            event.stopEvent();
            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }

            var sel = this.getSelectionModel();

            if (!sel.hasSelection()) {
                sel.selectRow(rowIndex, false);
            }

            var showRemoveMenuItem = false;

            var records = sel.getSelections();

            for (var i = 0; i < records.length; i++) {
                var record = records[i];

                // if at least one record can be removed - set the flag
                if (record.get('isPresent') === false) {
                    showRemoveMenuItem = true;
                    break;
                }
            }

            var menuItem = new Ext.menu.Item({
                text: _("Rename meter"),
                icon: 'images/page_white_edit.png',
                handler: this.editDevice.createDelegate(this, rowIndex, true)
            });

            this.contextMenu.add(menuItem);

            var menuItem = new Ext.menu.Item({
                text: _("Rescan circuit"),
                icon: 'images/page_white_edit.png',
                handler: this.rescanCircuit.createDelegate(this, rowIndex, true)
            });
            this.contextMenu.add(menuItem);

            if (showRemoveMenuItem === true) {
                var menuItem = new Ext.menu.Item({
                    text: _("Remove meter"),
                    icon: 'images/delete.png',
                    handler: this.removeMeter.createDelegate(this, rowIndex, true)
                });
                this.contextMenu.add(menuItem);
            }

            var record = sel.getSelected();

            if (dSS.config.features.dsmStatistics === true) {
                var subMenu = new Ext.menu.Menu({});

                if (record.get('isPresent') === true) {
                    var menuItem = new Ext.menu.Item({
                        text: _("Meter statistics"),
                        icon: 'images/application_osx_terminal.png',
                        handler: this.dsmStatistics.createDelegate(this, rowIndex, true)
                    });
                    subMenu.add(menuItem);
                }

                var menuItem = new Ext.menu.Item({
                    text: _("Abort meter statistics session"),
                    icon: 'images/application_osx_terminal.png',
                    handler: this.abortStatisticsRound.createDelegate(this, rowIndex, true)
                });
                subMenu.add(menuItem);

                var menuItem = new Ext.menu.Item({
                    text: _("Statistics"),
                    icon: 'images/application_osx_terminal.png',
                    menu: subMenu
                });
                this.contextMenu.add(menuItem);
            }

            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
        this);

        rowSelect.on('rowselect', function(selModel, rowIndex, dataRecord) {
            var store = dSS.data.Loader.getDeviceStore();
            store.filterBy(function(record) {
                if (record.get('meterDSID') === dataRecord.get('dsid')) {
                    return true;
                }
                return false;
            });
        },
        this);
    },
    buildTopToolbar: function() {
        return [
        {
            cls: 'zonePanelMenu',
            menu: [
            {
                text: _("Reload"),
                handler: this.reload,
                scope: this
            },
            {
                text: _("Rescan"),
                handler: this.rescan,
                scope: this
            }]
        },
        '->', {
            iconCls: 'copyToClipboardToolbarIcon',
            handler: this.showCopyWindow,
            tooltip: _("Show in copy to cliboard view"),
            scope: this
        },
        {
            iconCls: 'reloadAction',
            handler: this.removeInactiveDevices,
            tooltip: _("Remove inactive devices"),
            scope: this
        },
        {
            iconCls: 'removeDeviceToolbarIcon',
            handler: this.removeInactiveMeters,
            tooltip: _("Remove inactive meters"),
            scope: this
        }];
    },

    editDevice: function(item, event, rowIndex) {
        var record = this.getStore().getAt(rowIndex);
        Ext.Msg.prompt(_("Rename meter"), _("Name") + ": ", function(btn, text) {
            if (text !== record.get('name')) {
                Ext.Ajax.request({
                    url: '/json/circuit/setName',
                    disableCaching: true,
                    method: "GET",
                    params: {
                        id: record.get('dsid'),
                        newName: text
                    },
                    success: function(result, request) {
                        try {
                            var jsonData = Ext.util.JSON.decode(result.responseText);
                            if (jsonData.ok) {
                                record.set('name', text);
                                record.commit();

                                var devStore = dSS.data.Loader.getDeviceStore();
                                devStore.each(
                                    function(devRecord) {
                                        if (devRecord.get('meterDSID') ===
                                                record.get('dsid')) {
                                            devRecord.set('modulatorName',
                                                           text);
                                            devRecord.commit();
                                        }
                                    }
                                );

                            } else {
                                Ext.MessageBox.alert(_("Error"), _("Could not rename meter"));
                            }
                        }
                        catch(err) {
                            Ext.MessageBox.alert(_("Error"), _("Could not rename meter"));
                        }
                    },
                    failure: function(result, request) {
                        Ext.MessageBox.alert(_("Error"), _("Could not rename meter"));
                    }
                });
            }
        },
        this, false, record.get('name'));
    },
    setInitialSelection: function() {
        if (!this.isVisible()) {
            return;
        }

        if (this.getStore().getCount() === 0) {
            this.getStore().clearFilter();
            return;
        }

        var record;
        var index;

        if (this.getSelectionModel().hasSelection()) {
            index = this.getStore().indexOf(this.getSelectionModel().getSelected());
            record = this.getSelectionModel().getSelected();
        } else {
            this.getView().focusRow(0);
            this.getSelectionModel().selectFirstRow();
            record = this.getStore().getAt(0);
            index = 0;
        }

        this.getSelectionModel().fireEvent('rowselect', this.getSelectionModel(), index, record);
    },
    rescanCircuit: function(item, event, rowIndex) {
        var record = this.getStore().getAt(rowIndex);
        Ext.Ajax.request({
            url: '/json/circuit/rescan',
            disableCaching: true,
            method: "GET",
            params: {
                id: record.get('dsid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        // there is no point in reloading here, since the call to rescan is non blocking.
                        // when we get here, the apartment structure has not changed yet and there is
                        // no way to get a notification when the rescan process has finished.
                        // dSS.data.Loader.load();
                    } else {
                        Ext.MessageBox.alert(_("Error"), _("Rescan failed"));
                    }
                }
                catch(err) {
                    Ext.MessageBox.alert(_("Error"), _("Rescan failed") + ": " + err);
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(_("Error"), _("Rescan failed"));
            }
        });
    },
    removeInactiveDevices: function() {
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(_("No meter selected"), _("Please select a meter from the list!"));
            return;
        }

        record = sel.getSelected();
        var store = dSS.data.Loader.getDeviceStore();

        Ext.Ajax.request({
            url: '/json/structure/removeInactiveDevices',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                id: record.get('dsid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        store.each(
                        function(rec) {
                            if (rec.get('meterDSID') === record.get('dsid')) {
                                if (rec.get('isPresent') === false) {
                                    store.remove(rec);
                                }
                            }
                        },
                        this);
                    } else {
                        Ext.MessageBox.alert(_("Could not remove"), jsonData.message);
                    }
                }
                catch(err) {
                    Ext.MessageBox.alert(_("Error"), _("Could send remove device command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(_("Error"), _("Could send remove device command to dSS"));
            }
        });

    },
    showCopyWindow: function() {
        var tsvwindow = new dSS.util.GridToSVWindow({
            gridPanel: this,
            windowWidth: 640,
            windowHeight: 120,
            separator: "\t"
        });
        tsvwindow.show(this);
    },

    dsmStatistics: function(item, event, rowIndex) {
        // we were called from the toolbar, so its possible that there is
        // no device selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.Msg.alert(_("No meter selected"), _("Please select a meter from the list!"));
            return;
        }

        records = sel.getSelections();

        var dsms = "";
        var dsmArray = [];
        for (var i = 0; i < records.length; i++) {
            if (records[i].get("isPresent") === false) {
                continue;
            }
            var dsm = {};
            dsm.dsid = records[i].get("dsid");
            dsm.name = records[i].get("name");
            dsm.busid = records[i].get("busId");
            dsmArray.push(dsm);
            dsms += dsm.dsid;
            if (i < records.length - 1) {
                dsms += ",";
            }
        }

        if (dsmArray.length === 0) {
            Ext.Msg.alert(_("No meters present"), _("Your selection contained only non present meters!"));
            return;
        }

        var dsmRecord = Ext.data.Record.create([
        {
            name: "dsid",
            mapping: "dsid"
        },
        {
            name: "name",
            mapping: "name"
        },
        {
            name: "busid",
            mapping: "busid"
        }]);

        var statsStore = new Ext.data.ArrayStore({
            autoDestroy: true,
            storeId: "statsStore",
            idIndex: 0,
            fields: dsmRecord,
            data: dsmArray
        });
        var now = new Date();
        var h_now = now.getHours();
        var m_now = now.getMinutes();

        m_now += 5;
        if (m_now >= 60) {
            h_now++;
            m_now = m_now - 60;
        }

        if (h_now >= 24) { 
            h_now = 0;
        }

        var sep = ":";

        if (m_now < 10) {
            sep = sep + "0";
        }

        var statsForm = new dSS.util.DeviceStatisticsForm({
            statisticsStore: statsStore,
            sep: sep,
            h_now: h_now,
            m_now: m_now
        });

        var win = new dSS.util.DSMStatisticsWindow({
            form: statsForm,
            dsm: dsms,
            maskElement: this.findParentByType(Ext.Viewport).el
        });

        win.show(this);
    },
    abortStatisticsRound: function() {
        Ext.Ajax.request({
            url: '/json/property/getInteger',
            disableCaching: true,
            method: 'GET',
            scope: this,
            params: {
                path: '/system/js/settings/dsmStatistics/session'
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        var old_session = jsonData.result.value;
                        var new_session = old_session + 1;
                        Ext.Ajax.request({
                            url: '/json/property/setInteger',
                            disableCaching: true,
                            method: 'GET',
                            scope: this,
                            params: {
                                path: '/system/js/settings/dsmStatistics/session',
                                value: new_session
                            },
                            success: function(result, request) {
                                try {
                                    var jsonData = Ext.util.JSON.decode(result.responseText);
                                    if (jsonData.ok) {
                                        Ext.MessageBox.alert(_("Information"), _("dSM statistics session") + " #" + old_session + " " + _("aborted") + ".");
                                    } else {
                                        Ext.MessageBox.alert(_("Could not set statistics session value"), jsonData.message);
                                    }
                                } catch(err) {
                                    Ext.MessageBox.alert(_("Error"), _("Could send command to dSS") + ": " + err);
                                }
                            },
                            failure: function(result, request) {
                                Ext.MessageBox.alert(_("Error"), _("Could command to dSS"));
                            }
                        });
                    } else {
                        Ext.MessageBox.alert(_("Could not get dSM statistics session value"), jsonData.message);
                    }
                } catch(err) {
                    Ext.MessageBox.alert(_("Error"), _("Could send command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(_("Error"), _("Could send command to dSS"));
            }
        });
    },
    removeMeter: function(item, event, rowIndex) {
        var records;

        // we were called from the toolbar, so its possible that there is
        // no meter selection yet
        var sel = this.getSelectionModel();
        if (!sel.hasSelection()) {
            Ext.MessageBox.alert(_("No meter selected"), _("Please select a meter from the list!"));
            return;
        }
        records = sel.getSelections();

        var skippedPresent = false;
        var notPresent = [];

        for (var i = 0; i < records.length; i++) {
            var record = records[i];

            // do not issue the ajax request if we know that it will fail
            if (record.get('isPresent') === true) {
                skippedPresent = true;
                continue;
            }

            notPresent.push(record);
        }

        if (notPresent.length > 0) {
            this.deleteMeter(notPresent);
        } else {
            if (skippedPresent === true) {
                Ext.MessageBox.alert(_("Warning"), _("Present meters can not be removed!"));
            }
        }
    },
    deleteMeter: function(records) {
        var store = this.getStore();
        if (records.length < 1) {
            this.el.unmask();
            return;
        }

        var record = records.shift();

        this.el.mask(_("Deleting meter") + ": " + record.get("dsid"));
        Ext.Ajax.request({
            url: '/json/apartment/removeMeter',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                dsid: record.get('dsid')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        store.remove(record);
                        this.deleteMeter(records);
                    } else {
                        this.el.unmask();
                        Ext.MessageBox.alert(_("Could not remove"), jsonData.message);
                    }
                }
                catch(err) {
                    this.el.unmask();
                    Ext.MessageBox.alert(_("Error"), _("Could send remove meter command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                this.el.unmask();
                Ext.MessageBox.alert(_("Error"), _("Could send remove meter command to dSS"));
            }
        });
    },
    removeInactiveMeters: function() {
        Ext.Ajax.request({
            url: '/json/apartment/removeInactiveMeters',
            disableCaching: true,
            method: "GET",
            scope: this,
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        dSS.data.Loader.load();
                    } else {
                        Ext.MessageBox.alert(_("Could not remove inactive meters"), jsonData.message);
                    }
                }
                catch(err) {
                    Ext.MessageBox.alert(_("Error"), _("Could send remove inactive meters command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(_("Error"), _("Could send remove inactive meters command to dSS"));
            }
        });
    },
    rescan: function() {
        this.ownerCt.ownerCt.zoneBrowser.rescan();
    },
    reload: function() {
        this.ownerCt.ownerCt.zoneBrowser.loadData();
    }
});

Ext.reg('dssmodulatorpanel', dSS.grid.ModulatorPanel);

