//= require <dSS/GetTextHelper>
//= require <dSS/util/PingEventWindow>
Ext.namespace("dSS", "dSS.util");

dSS.util.DeviceStatisticsWindow = Ext.extend(Ext.Window, {
    initComponent: function() {
        Ext.apply(this, {
            layout: "fit",
            width: 380,
            height: 475,
            items: [this.form],
            buttons: [
            {
                text: _("OK"),
                formBind: true,
                scope: this,
                handler: function() {
                    if (!this.form.getForm().isValid()) {
                        return;
                    }

                    var param_time = "";

                    if (this.form.getForm().getValues().time !== undefined) {

                        var current = new Date();
                        var d = new Date();
                        var hours = parseInt(this.form.getForm().getValues().time.split(':')[0], 10);
                        var minutes = parseInt(this.form.getForm().getValues().time.split(':')[1], 10);
                        d.setHours(hours);
                        d.setMinutes(minutes);

                        if ((hours * 60 + minutes) <= (current.getHours() * 60 + current.getMinutes())) {
                            Ext.MessageBox.alert(_("Error"), _("Event time must be in the future!"));
                            return;
                        }

                        var pad = function(n) {
                            return n < 10 ? '0' + n: n;
                        };

                        param_time = "time=" + d.getFullYear() + pad(d.getMonth() + 1) + pad(d.getDate()) + 'T' + pad(d.getHours()) + pad(d.getMinutes()) + pad(d.getSeconds()) + (d.getTimezoneOffset() / 60) + ";";
                    }

                    var param = param_time + "delay=" + this.form.getForm().getValues().delay + ";repeat=" + this.form.getForm().getValues().repeat + ";dsid=" + this.dsids;

                    // close statistics form dialog
                    this.maskElement.unmask();
                    this.close();

                    this.raiseStatisticsEvent(param, param_time.length);
                }
            },
            {
                text: _("Cancel"),
                scope: this,
                handler: function() {
                    this.close();
                    this.maskElement.unmask();
                }
            }]
        });

        dSS.util.DeviceStatisticsWindow.superclass.initComponent.apply(this, arguments);

        this.maskElement.mask();
    },
    raiseStatisticsEvent: function(param, later) {
        Ext.Ajax.request({
            url: '/json/event/raise',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                name: "device_statistics",
                parameter: param
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        var title = _("Statistics result information");
                        var message = _("The statistics results can be viewed in the js plugin log files section");
                        var file = dSS.config.settings["deviceStatistics"].logfile;

                        if (file !== undefined) {
                            message = message + _(" or downloaded here: ") + "<span class=\"x-green-button-link\">" + "<a target=\"_blank\" title=\"" + _("Right click and select Save Link As to download" + " the log file.") + "\" href=\"/download/" + file + "\">" + file + "</a></span>";
                        } else {
                            message = message + ".";
                        }

                        Ext.MessageBox.show({
                            title: title,
                            msg: message,
                            buttons: Ext.Msg.OK,
                            minWidth: 300
                        });
                    } else {
                        Ext.MessageBox.alert(_("Statistics error"), jsonData.message);
                    }
                }
                catch(err) {
                    Ext.MessageBox.alert(_("Error"), _("Could send statistics command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(_("Error"), _("Could send stat command to dSS"));
            }
        });

    }
});

Ext.reg("dssutildevicestatisticswindow", dSS.util.DeviceStatisticsWindow);

