//= require <dSS/GetTextHelper>
//= require <dSS/util/PingEventWindow>
Ext.namespace("dSS", "dSS.util");

dSS.util.ExtendedPingWindow = Ext.extend(Ext.Window, {
    initComponent: function() {
        Ext.apply(this, {
            layout: "fit",
            width: 380,
            height: 465,
            items: [this.form],
            buttons: [
            {
                text: _("OK"),
                formBind: true,
                scope: this,
                handler: function() {
                    if (!this.form.getForm().isValid()) {
                        return;
                    }

                    var param_time = "";

                    if (this.form.getForm().getValues().time !== undefined) {
                        var current = new Date();
                        var d = new Date();
                        var hours = parseInt(this.form.getForm().getValues().time.split(':')[0], 10);
                        var minutes = parseInt(this.form.getForm().getValues().time.split(':')[1], 10);
                        d.setHours(hours);
                        d.setMinutes(minutes);

                        if ((hours * 60 + minutes) <= (current.getHours() * 60 + current.getMinutes())) {
                            Ext.MessageBox.alert(_("Error"), _("Event time must be in the future!"));
                            return;
                        }

                        var pad = function(n) {
                            return n < 10 ? '0' + n: n;
                        };

                        param_time = "time=" + d.getFullYear() + pad(d.getMonth() + 1) + pad(d.getDate()) + 'T' + pad(d.getHours()) + pad(d.getMinutes()) + pad(d.getSeconds()) + (d.getTimezoneOffset() / 60) + ";";
                    }

                    var param = param_time + "delay=" + this.form.getForm().getValues().delay + ";repeat=" + this.form.getForm().getValues().repeat + ";dsid=" + this.dsids;

                    // close ping form dialog
                    this.maskElement.unmask();
                    this.close();

                    // if time was specified, we will ping and thus see results
                    // later, this will be done via logs
                    //
                    // however, if we are pinging "now", we will subscribe
                    // to the ping result events and display the results
                    // in real time
                    if (param_time.length > 0) {
                        this.raisePingEvent(param, param_time.length);
                    } else {
                        Ext.Ajax.request({
                            url: '/json/event/subscribe',
                            disableCaching: true,
                            method: "GET",
                            scope: this,
                            params: {
                                subscriptionID: 333,
                                name: "ping_result"
                            },
                            success: function(result, request) {
                                try {
                                    var jsonData = Ext.util.JSON.decode(result.responseText);
                                    if (jsonData.ok) {
                                        this.raisePingEvent(param, param_time.length);
                                    } else {
                                        Ext.MessageBox.alert(_("Ping subscription error"), jsonData.message);
                                    }
                                } catch(err) {
                                    Ext.MessageBox.alert(_("Error"), _("Could send subscribe command to dSS") + ": " + err);
                                }
                            },
                            failure: function(result, request) {
                                Ext.MessageBox.alert(_("Error"), _("Could send subscribe command to dSS"));
                            }
                        });
                    }
                }
            },
            {
                text: _("Cancel"),
                scope: this,
                handler: function() {
                    this.close();
                    this.maskElement.unmask();
                }
            }]
        });

        dSS.util.ExtendedPingWindow.superclass.initComponent.apply(this, arguments);

        this.maskElement.mask();
    },
    raisePingEvent: function(param, later) {
        Ext.Ajax.request({
            url: '/json/event/raise',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                name: "ping",
                parameter: param
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        if (later > 0) {
                            var title = _("Ping result information");
                            var message = _("You specified a ping event in the future, the results will be available in the log files section as soon as the ping starts.");
                            Ext.MessageBox.show({
                                title: title,
                                msg: message,
                                buttons: Ext.Msg.OK,
                                minWidth: 300
                            });
                        } else {
                            var pingResult = new dSS.util.PingEventWindow({
                                windowWidth: 640,
                                windowHeight: 480,
                                maskElement: this.maskElement
                            });
                            pingResult.show(this);
                        }
                    } else {
                        Ext.MessageBox.alert(_("Ping error"), jsonData.message);
                    }
                }
                catch(err) {
                    Ext.MessageBox.alert(_("Error"), _("Could send ping command to dSS") + ": " + err);
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(_("Error"), _("Could send ping command to dSS"));
            }
        });

    }
});

Ext.reg("dssutilextendedpingwindow", dSS.util.ExtendedPingWindow);

