//= require <dSS/GetTextHelper>
Ext.namespace("dSS", "dSS.util");

dSS.util.PingEventWindow = Ext.extend(Ext.Window, {
    constructor: function(config) {
        // special session id used throughout the code
        this.sid = 333;

        this.stopRequests = false;

        this.parameter = config.parameter;

        this.maskElement = config.maskElement;

        this.eventRecord = Ext.data.Record.create([
        {
            name: "logged to",
            mapping: "logfile"
        },
        {
            name: "dsid",
            mapping: "dsid"
        },
        {
            name: "timestamp",
            mapping: "timestamp"
        },
        {
            name: "receive",
            mapping: "receive"
        },
        {
            name: "send",
            mapping: "send"
        },
        {
            name: "name",
            mapping: "name"
        },
        {
            name: "reqindex",
            mapping: "reqindex"
        }]);

        this.pingEventStore = new Ext.data.ArrayStore({
            autoDestroy: true,
            storeId: "pingEventStore",
            idIndex: 0,
            fields: this.eventRecord
        });

        this.resultPanel = new Ext.grid.GridPanel({
            /*
                colModel: new Ext.grid.ColumnModel({
                    defaults: { sortable : false }
                }),*/
            columns: [
            {
                header: _("timestamp"),
                dataIndex: "timestamp",
                xtype: "datecolumn",
                id: "timestamp",
                format: "Y-m-d\\&\\n\\b\\s\\p\\;H:i:s",
                sortable: true
            },
            {
                header: "#",
                dataIndex: "reqindex",
                id: "dsid",
                sortable: false,
                width: 35
            },
            {
                header: _("dsid"),
                dataIndex: "dsid",
                id: "dsid",
                sortable: false
            },
            {
                header: _("send"),
                dataIndex: "send",
                sortable: false,
                width: 50
            },
            {
                header: _("receive"),
                dataIndex: "receive",
                sortable: false,
                width: 50
            },
            {
                header: _("name"),
                dataIndex: "name",
                sortable: false
            },
            {
                header: _("logged to"),
                dataIndex: "logfile",
                sortable: false,
                renderer: function(val) {
                    if (val !== null) {
                        return "<span class=\"x-green-button-link\"><a target=\"_blank\" title=\"" + _("Right click and select Save Link As to download the log file.") + "\" href=\"/download/" + val + "\">" + val + "</a></span>";
                    }
                    return val;
                }
            }],
            autoExpandColumn: "timestamp",
            store: this.pingEventStore,
            disableSelection: true,
            frame: false,
            forceFit: true,
            stripeRows: true,
            region: "center",
            autoScroll: true,
            layout: "fit",
            viewConfig: {
                autoFill: true
            }
        });

        Ext.apply(this, {
            layout: "border",
            title: _("Ping results"),
            width: config.windowWidth,
            height: config.windowHeight,
            items: [this.resultPanel],
            buttons: [
            {
                text: _("Close"),
                handler: this.close.createDelegate(this)
            }]
        });

        dSS.util.PingEventWindow.superclass.constructor.apply(this, arguments);
        this.maskElement.mask();

        this.on("afterrender", function(component) {
            this.getNextEvent();
        },
        this);

        this.on("beforedestroy", function(component) {
            this.stopRequests = true;
            this.maskElement.unmask();
        },
        this);
    },

    getNextEvent: function() {
        if (this.stopRequests) {
            this.stopRequests = false;
            Ext.Ajax.request({
                url: '/json/event/unsubscribe',
                disableCaching: true,
                method: "GET",
                scope: this,
                params: {
                    subscriptionID: 333,
                    name: "ping_result"
                },
                success: function(result, request) {
                    try {
                        var jsonData = Ext.util.JSON.decode(result.responseText);
                        if (!jsonData.ok) {
                            Ext.MessageBox.alert(_("Ping event unsubscribe error"), jsonData.message);
                        }
                    } catch(err) {
                        Ext.MessageBox.alert(_("Error"), _("Could not send unsubscribe command to dSS") + ": " + err);
                    }
                },
                failure: function(result, request) {
                    Ext.MessageBox.alert(_("Error"), _("Could not send unsubscribe command to dSS"));
                }
            });

            return;
        }

        Ext.Ajax.request({
            url: "/json/event/get",
            method: "GET",
            scope: this,
            params: {
                subscriptionID: this.sid,
                timeout: "2000"
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        for (var i = 0; i < jsonData.result.events.length; i++) {
                            var r = new this.eventRecord(
                            jsonData.result.events[i].properties);
                            this.pingEventStore.add([r]);
                        }

                        this.getNextEvent();

                    } else {
                        Ext.MessageBox.alert(_("Error"), _("Could not get ping event from dSS"));
                    }
                } catch(err) {
                    if (this.data !== null) { // window was closed 
                        Ext.MessageBox.alert(_("Error"), _("Could send get command to dSS") + ": " + err);
                    }
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(_("Error"), _("Could send get command to dSS"));
            }
        });
    }
});

Ext.reg('dssutilpingeventwindow', dSS.util.PingEventWindow);

