Ext.namespace("dSS", "dSS.util");

dSS.util.decode16 = function(version) {
    var low =  (version & 0x000000ff);
    var high = (version & 0x0000ff00) >> 8;
    return "" + high + "." + low;
};

dSS.util.decode32 = function(version) {
    var v1 = (version & 0xff000000) >> 24;
    var v2 = (version & 0x00ff0000) >> 16;
    var v3 = (version & 0x0000ff00) >> 8;
    var v4 = (version & 0x000000ff);

    return "" + v1 + "." + v2  + "." + v3 + "." + v4;
};

dSS.util.parseFunctionID = function(functionID) {
    var dSclass = (functionID & 0xf000) >> 12;
    var dSsubclass = (functionID & 0xfc0) >> 6;
    var dSfuncModule = (functionID & 0x3f);

    return [dSclass, dSsubclass, dSfuncModule];
};

dSS.util.parseProductID = function(productID) {
    var deviceType = (productID >> 9) & 0x1ff; // (2^9) - 1
    var deviceNumber = productID & 0x3ff; // (2^10) - 1
    return [deviceType, deviceNumber];
};

dSS.util.deviceStringFromProduct = function(functionID, productID) {
    var fid = dSS.util.parseFunctionID(functionID);
    var devShort = dSS.config.deviceColors[fid[0]];
    if (devShort === undefined) {
        devShort = "";
    } else {
        devShort = devShort + "-";
    }

    var result = dSS.util.parseProductID(productID);
    if (result.length < 2) {
        return "";
    }

    var devType = dSS.config.deviceTypes[result[0]];
    if (devType === undefined) {
        devType = "??";
    }

    var devNumber = result[1];
    if (devNumber === undefined) {
        devNumber = "???";
    }

    return devShort + devType + " " + result[1];

};

dSS.util.dec2hex = function(d) {
  var hD = '0123456789ABCDEF';
  var h = hD.substr(d & 15, 1);
  while (d > 15) {
    d>>=4;
    h = hD.substr(d & 15, 1) + h;
  }
  return h;
};

dSS.util.callScene = function(dsid, sceneNumber) {
    Ext.Ajax.request({
        url: '/json/device/callScene',
        disableCaching: true,
        method: "GET",
        params: {
            dsid: dsid,
            sceneNumber: sceneNumber
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.MessageBox.alert(_("Could not call scene") + " " + sceneNumber, jsonData.message);
                }
            } catch(err) {
                Ext.MessageBox.alert(_("Error"), _("Could not send callScene command to dSS") + ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.MessageBox.alert(_("Error"), _("Could not send callScene command to dSS"));
        }
    });
};

dSS.util.callSceneOnZone = function(zoneId, sceneNumber, groupId) {
    Ext.Ajax.request({
        url: '/json/zone/callScene',
        disableCaching: true,
        method: "GET",
        params: {
            id: zoneId,
            sceneNumber: sceneNumber,
            groupID: groupId
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.MessageBox.alert(_("Could not call scene") + " " + sceneNumber, jsonData.message);
                }
            } catch(err) {
                Ext.MessageBox.alert(_("Error"), _("Could not send callScene command to dSS") + ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.MessageBox.alert(_("Error"), _("Could not send callScene command to dSS"));
        }
    });
};

dSS.util.saveSceneOnZone = function(zoneId, sceneNumber, groupId) {
    Ext.Ajax.request({
        url: '/json/zone/saveScene',
        disableCaching: true,
        method: "GET",
        params: {
            id: zoneId,
            sceneNumber: sceneNumber,
            groupID: groupId
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.MessageBox.alert(_("Could not save scene") + " " + sceneNumber, jsonData.message);
                }
            } catch(err) {
                Ext.MessageBox.alert(_("Error"), _("Could not send saveScene command to dSS") + ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.MessageBox.alert(_("Error"), _("Could not send saveScene command to dSS"));
        }
    });
};

dSS.util.setValueOnDevice = function(dsid, value) {
    Ext.Ajax.request({
        url: '/json/device/setValue',
        disableCaching: true,
        method: "GET",
        params: {
            dsid: dsid,
            value: value
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.MessageBox.alert(_("Could not set value") + ": " + jsonData.message);
                }
            } catch(err) {
                Ext.MessageBox.alert(_("Error"), _("Could not send setValue command to dSS") + ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.MessageBox.alert(_("Error"), _("Could not send setValue command to dSS"));
        }
    });

};

